generateFunction_DisplayFunctions :: Int -> [(String,String)] ->
    [String] -> ShowS
generateFunction_DisplayFunctions maxLen flist terms = 
   showString "guiDisplayFunctions :: " .
   showString "[(Int,String,String,[(String,String)])]\n" .
   showString "guiDisplayFunctions = [ (0,[],[],[])\n" .
   foldr (.) id [ left . showString ", (" . shows i . showString ", " .
                  format fun . 
                  showString ", asmShow (" . showString fun .
                  showString " " . showString sig .
                  showString "),asmShowAssocs (" . showString fun .
                  showString " " . showString sig .
                  showString "))\n"
                | (i,(fun,sig)) <- zip [1..] flist
                ] .
   foldr (.) id [ left . showString ", (" .shows i . showString ", " .
                  format term . 
                  showString ", show (" . showString term .
                  showString "),[])\n"
                | (i,term) <- zip [1 + length flist ..] terms
                ] . left . showString "]\n\n"
 where left = showString "  "
       format s = showString "\"" .
                  showString (take maxLen s) .
                  showString (copy (max 0 (maxLen-length s)) ' ') .
                  showString "  \""

generateFunction_Main :: String -> String -> Bool -> Bool -> String  -> ShowS
generateFunction_Main main title history update fontStr = 
     showString main . showString " :: IO ()\n" .
     showString main . showString " = general_guiMain " .
     shows title . shows history . showString " " .
     shows update . showString " " . shows fontStr . showString " " .
     showString " guiDisplayFunctions guiUsedRules guiUsedConds\n\n"

generateFunction_UsedRules :: [String] -> ShowS
generateFunction_UsedRules [] = 
     showString "guiUsedRules :: [(String, Rule ())]\n" .
     showString "guiUsedRules = []\n\n"
generateFunction_UsedRules rules =
     showString "guiUsedRules :: [(String, Rule ())]\n" .
     showString "guiUsedRules = " .
     showString "[" .
     foldr1 (\l r -> l . showString ", " . r)
            (map (\r -> showString "(" . shows r . showString ", " .
                        showString r . showString ")") rules) .
     showString "]\n\n"

generateFunction_UsedConds :: [String] -> ShowS
generateFunction_UsedConds [] = 
     showString "guiUsedConds :: [(String, Bool)]\n" .
     showString "guiUsedConds = []\n\n"
generateFunction_UsedConds conds =
     showString "guiUsedConds :: [(String, Bool)]\n" .
     showString "guiUsedConds = " .
     showString "[" .
     foldr1 (\l r -> l . showString ", " . r)
            (map (\r -> showString "(" . shows r . showString ", " .
                        showString r . showString ")") conds) .
     showString "]\n\n"


generateGUI :: String -> IO ()
generateGUI guilib = start $ do
   configs <- liftIO (getConfiguration "gui.config")
   let dynfuns = config_values configs "FUNS"
       out     = cfgGetKey configs "FILES" "output"    "guiMain.gs"
       main    = cfgGetKey configs "GUI"   "main"      "guiMain"
       title   = cfgGetKey configs "GUI"   "title"     "main"
       rules   = config_values configs "RULES"
       terms   = config_values configs "TERMS"
       conds   = config_values configs "CONDS"
       dflt    = config_values configs "DEFAULT"
       fontStr = cfgGetKey configs "DEFAULT" "font" "7x14"
       history = "history" `elem` dflt
       update  = "update"  `elem` dflt
   let flist = map (break (==' ')) dynfuns
       m     = maximum (map (length . fst) flist
                       ++ map length terms)
   writeFile out
     ( showString (openfile guilib)
     . generateFunction_DisplayFunctions m flist terms
     . generateFunction_UsedRules rules
     . generateFunction_UsedConds conds
     . generateFunction_Main main title history update fontStr $ "")
   putStrLn "\nGUI successfully created"
   quit
